/* This file is part of Cloudy and is copyright (C) 1978-2005 by Gary J. Ferland.
 * For conditions of distribution and use, see copyright notice in license.txt */
/*mole_H2_LTE sets Boltzmann factors and LTE unit population of large H2 molecular */
/*H2_init - called to initialize things from cdInit */
/*H2_Zero zero out vars in the large H2 molecule, called from zero 
 * before any commands are parsed */
/* H2_zero_pops_too_low - zero out some H2 variables if we decide not to compute
 * the full sim, called by H2_LevelPops*/
#include "cddefines.h" 
#include "phycon.h" 
#include "mole.h" 
#include "hmi.h" 
#include "taulines.h" 
#include "h2_priv.h" 
#include "h2.h" 

/* H2_zero_pops_too_low - zero out some H2 variables if we decide not to compute
 * the full sim, called by H2_LevelPops*/
void H2_zero_pops_too_low( void )
{

	long int iElec, iElecHi, iElecLo, iVib , iVibLo , iVibHi ,
		iRot , iRotHi , iRotLo;

#	ifdef DEBUG_FUN
	fputs( "<+>H2_zero_pops_too_low()\n", debug_fp );
#	endif

	/* >>chng 05 jan 26, add this block to set populations to lte value */
	for( iElec=0; iElec<mole.n_h2_elec_states; ++iElec )
	{
		for( iVib=0; iVib<=nVib_hi[iElec]; ++iVib )
		{
			for( iRot=Jlowest[iElec]; iRot<=nRot_hi[iElec][iVib]; ++iRot )
			{
				/* LTE populations are for unit H2 density, so need to multiply
					* by total H2 density */
				H2_old_populations[iElec][iVib][iRot] = 
					(float)H2_populations_LTE[iElec][iVib][iRot]*hmi.H2_total;
				H2_populations[iElec][iVib][iRot] = H2_old_populations[iElec][iVib][iRot];
			}
		}
	}
	/* zero everything out - loop over all possible lines */
	/* >>chng 05 jan 26, set to lte values, since we still need to accumulate Lyman line
		* optical depths to have correct self-shielding when large h2 does come on */
	for( iElecHi=0; iElecHi<mole.n_h2_elec_states; ++iElecHi )
	{
		pops_per_elec[iElecHi] = 0.;
		for( iVibHi=0; iVibHi<=nVib_hi[iElecHi]; ++iVibHi )
		{
			pops_per_vib[iElecHi][iVibHi] = 0.;
			for( iRotHi=Jlowest[iElecHi]; iRotHi<=nRot_hi[iElecHi][iVibHi]; ++iRotHi )
			{
				long int lim_elec_lo = 0;
				/* now the lower levels */
				/* NB - iElecLo the lower electronic level is only X 
					* we don't consider excited elec to excit elec trans here, since we are only 
					* concerned with excited electronic levels as a photodissociation process
					* code exists to relax this assumption - simply change following to iElecHi */
				for( iElecLo=0; iElecLo<=lim_elec_lo; ++iElecLo )
				{
					/* want to include all vib states in lower level if different elec level,
					* but only lower vib levels if same elec level */
					long int nv = nVib_hi[iElecLo];
					if( iElecLo==iElecHi )
						nv = iVibHi;
					for( iVibLo=0; iVibLo<=nv; ++iVibLo )
					{
						long nr = nRot_hi[iElecLo][iVibLo];
						if( iElecLo==iElecHi && iVibHi==iVibLo )
							nr = iRotHi-1;

						for( iRotLo=Jlowest[iElecLo]; iRotLo<=nr; ++iRotLo )
						{
							if( lgH2_line_exists[iElecHi][iVibHi][iRotHi][iElecLo][iVibLo][iRotLo] )
							{

								/* population of lower level */
								/* >>chng 05 jan 26, set to lte value */
								H2Lines[iElecHi][iVibHi][iRotHi][iElecLo][iVibLo][iRotLo].PopLo = 
									H2_populations[iElecLo][iVibLo][iRotLo];

								/* population of upper level */
								H2Lines[iElecHi][iVibHi][iRotHi][iElecLo][iVibLo][iRotLo].PopHi = 
									H2_populations[iElecHi][iVibHi][iRotHi];

								/* population of lower level with correction for stim emission */
								H2Lines[iElecHi][iVibHi][iRotHi][iElecLo][iVibLo][iRotLo].PopOpc = 
									H2Lines[iElecHi][iVibHi][iRotHi][iElecLo][iVibLo][iRotLo].PopLo - 
									H2Lines[iElecHi][iVibHi][iRotHi][iElecLo][iVibLo][iRotLo].PopHi*
									H2Lines[iElecHi][iVibHi][iRotHi][iElecLo][iVibLo][iRotLo].gLo / 
									H2Lines[iElecHi][iVibHi][iRotHi][iElecLo][iVibLo][iRotLo].gHi;

								/* following two heat exchange excitation, deexcitation */
								H2Lines[iElecHi][iVibHi][iRotHi][iElecLo][iVibLo][iRotLo].cool = 0.;
								H2Lines[iElecHi][iVibHi][iRotHi][iElecLo][iVibLo][iRotLo].heat = 0.;

								/* intensity of line */
								H2Lines[iElecHi][iVibHi][iRotHi][iElecLo][iVibLo][iRotLo].xIntensity = 0.;

								H2Lines[iElecHi][iVibHi][iRotHi][iElecLo][iVibLo][iRotLo].phots = 0.;
								H2Lines[iElecHi][iVibHi][iRotHi][iElecLo][iVibLo][iRotLo].ots = 0.;
								H2Lines[iElecHi][iVibHi][iRotHi][iElecLo][iVibLo][iRotLo].ColOvTot = 0.;
							}
						}
					}
				}
			}
		}
	}
	hmi.H2_photodissoc_BigH2_H2s = 0.;
	hmi.H2_photodissoc_BigH2_H2g = 0.;
	hmi.HeatH2Dish_BigH2 = 0.;
	hmi.HeatH2Dexc_BigH2 = 0.;
	hmi.deriv_HeatH2Dexc_BigH2 = 0.;
	hmi.H2_Solomon_dissoc_rate_BigH2_H2g = 0.;
	hmi.H2_Solomon_dissoc_rate_BigH2_H2s = 0.;
	hmi.H2_H2g_to_H2s_rate_BigH2 = 0.;

#	ifdef DEBUG_FUN
	fputs( " <->H2_zero_pops_too_low()\n", debug_fp );
#	endif

	return;
}

/*mole_H2_LTE sets Boltzmann factors and LTE unit population of large H2 molecular */
void mole_H2_LTE( void )
{
	/* used to recall the temperature used for last set of Boltzmann factors */
	static double TeUsedBoltz = -1.;
	double part_fun;
	long int iElec , iVib , iRot;

#	ifdef DEBUG_FUN
	fputs( "<+>mole_H2_LTE()\n", debug_fp );
#	endif

	/* do we need to update the boltzmann factors and unit LTE populations? */
	/*lint -e777 float test equality */
	if( phycon.te != TeUsedBoltz )
	/*lint +e777 float test equality */
	{
		part_fun = 0.;
		TeUsedBoltz = phycon.te;
		/* loop over all levels setting H2_Boltzmann and deriving partition function */
		for( iElec=0; iElec<mole.n_h2_elec_states; ++iElec )
		{
			for( iVib=0; iVib<=nVib_hi[iElec]; ++iVib )
			{
				for( iRot=Jlowest[iElec]; iRot<=nRot_hi[iElec][iVib]; ++iRot )
				{
					H2_Boltzmann[iElec][iVib][iRot] = 
						/* energy is relative to lowest level in the molecule, v=0, J=0,
						 * so Boltzmann factor is relative to this level */
						sexp( energy_wn[iElec][iVib][iRot] / phycon.te_wn );
					/* sum the partition funciton - boltzmann factor times statistical weight */
					part_fun += H2_Boltzmann[iElec][iVib][iRot] * H2_stat[iElec][iVib][iRot];
					ASSERT( part_fun > 0 );
				}
			}
		}
		/* have partition function, set H2_populations_LTE (populations for unit H2 density) */
		for( iElec=0; iElec<mole.n_h2_elec_states; ++iElec )
		{
			for( iVib=0; iVib<=nVib_hi[iElec]; ++iVib )
			{
				for( iRot=Jlowest[iElec]; iRot<=nRot_hi[iElec][iVib]; ++iRot )
				{
					/* these are the H2 LTE populations for a unit H2 density -
					 * these populations will sum up to unity */
					H2_populations_LTE[iElec][iVib][iRot] = 
						H2_Boltzmann[iElec][iVib][iRot] * 
						H2_stat[iElec][iVib][iRot] / part_fun;
					/*if( iElec==0 && iVib < 2)
						fprintf(ioQQQ,"DEBUG lte pop\t%i\t%i\t%e\n",
						iVib,iRot,H2_populations_LTE[iElec][iVib][iRot]*hmi.H2_total);*/
				}
			}
		}
		if( mole.lgH2_TRACE >= mole.lgH2_trace_full ) 
			fprintf(ioQQQ,
			"mole_H2_LTE set H2_Boltzmann factors, T=%.2f, partition function is %.2f\n",
			phycon.te,
			part_fun);
	}


#	ifdef DEBUG_FUN
	fputs( " <->mole_H2_LTE()\n", debug_fp );
#	endif

	return;

}

/*H2_init - called to initialize things from cdInit */
void H2_Init(void)
{

#	ifdef DEBUG_FUN
	fputs( "<+>H2_Init()\n", debug_fp );
#	endif

	/* the number of electronic quantum states to include.
	 * To do both Lyman and Werner bands want nelec = 3,
	 * default is to do all bands included */
	mole.n_h2_elec_states = N_H2_ELEC;
	h2.nCallH2_this_zone = 0;

#	ifdef DEBUG_FUN
	fputs( " <->H2_Init()\n", debug_fp );
#	endif

	return;
}


/*H2_Reset called by IterRestart to reset variables that are needed after an iteration */
void H2_Reset( void )
{

#	ifdef DEBUG_FUN
	fputs( "<+>H2_Reset()\n", debug_fp );
#	endif

	if(mole.lgH2_TRACE) 
		fprintf(ioQQQ,
		"\n***************H2_Reset called, resetting nCallH2_this_iteration, zone %.2f iteration %li\n", 
		fnzone,
		iteration );

	/* number of times large molecules evaluated in this iteration,
	 * is FALSE if never evaluated, on next evaluation will start with lte populations */
	nCallH2_this_iteration = 0;

	/* these remember the largest and smallest factors needed to
	 * renormalize the H2 chemistry */
	h2.renorm_max = 1.;
	h2.renorm_min = 1.;

	/* counters used by H2_itrzn to find number of calls of h2 per zone */
	nH2_pops  = 0;
	nH2_zone = 0;
	/* this is used to establish zone number for evaluation of number of levels in matrix */
	nzone_nlevel_set = 0;

	nzoneAsEval = -1;
	iterationAsEval = -1; 

#	ifdef DEBUG_FUN
	fputs( " <->H2_Reset()\n", debug_fp );
#	endif

	return;

}

/*H2_Zero zero out vars in the large H2 molecule, called from zero 
 * before any commands are parsed 
 * NB - this routine is called before space allocated - must not zero out
 * any allocated arrays */
void H2_Zero( void )
{

#	ifdef DEBUG_FUN
	fputs( "<+>H2_Zero()\n", debug_fp );
#	endif

	/* this is the smallest ratio of H2/H where we will bother with the large H2 molecule
	 * this value was chosen so that large mole used at very start of TH85 standard pdr,
	 * NB - this appears in headinfo and must be updated there if changed here */
	/* >>chng 03 jun 02, from 1e-6 to 1e-8 - in orion veil large H2 turned on half way
	 * across, and solomon process was very fast since all lines optically thin.  correct
	 * result has some shielding, so reset to lower value so that H2 comes on sooner. */
	mole.H2_to_H_limit = 1e-8;

	/* these are used to set trace levels of output by options on atom h2 trace command 
	 * first is minimum level of trace, keyword is FINAL */
	mole.lgH2_trace_final = 1;
	/* intermediate level of trace, info per iteration, key ITERATION */
	mole.lgH2_trace_iterations = 2;
	/* full trace, keyword is FULL */
	mole.lgH2_trace_full = 3;
	/* print matrices used in solving X */
	mole.lgH2_trace_matrix = 4;

	h2.lgH2ON = FALSE;
	mole.lgH2_TRACE = FALSE;
	/* flag to force using LTE level populations */
	mole.lgH2_LTE = FALSE;

	/* this is number of electronic levels to include in the print and punch output 
	 * changed with the PRINT LINE H2 ELECTRONIC and PUNCH H2 commands 
	 * by default only include X */
	h2.nElecLevelOutput = 1;

	/* counters used by H2_itrzn to find number of calls of h2 per zone */
	nH2_pops  = 0;
	nH2_zone = 0;
	/* this is used to establish zone number for evaluation of number of levels in matrix */
	nzone_nlevel_set = 0;

	/* option to scramble collision data */
	mole.lgH2_NOISE = FALSE;
	mole.lgH2_NOISECOSMIC = FALSE; 

	/* option to turn off or on gbar collisions of the collision rate,
	 * default is to have it on */
	/* turn mole.lgColl_gbar on/off with atom h2 gbar on off */
	mole.lgColl_gbar = TRUE;

	/* include collision rates that come from real calculations,
	 * off with atom h2 collisions off command */
	mole.lgColl_deexec_Calc = TRUE;

	mole.lgColl_dissoc_coll = TRUE;

	/* option to turn off H2 - grain collision & deexcitation,
	 * atom h2 grain collision on/off */
	mole.lgH2_grain_deexcitation = FALSE;

	/* option to turn off ortho-para collisions, command ATOM H2 COLLISIONS ORTHO PARA OFF */
	mole.lgH2_ortho_para_coll_on = TRUE;

	/* these remember the largest and smallest factors needed to
	 * renormalize the H2 chemistry */
	h2.renorm_max = 1.;
	h2.renorm_min = 1.;

	nCallH2_this_iteration = 0;
	h2.ortho_density = 0.;
	h2.para_density = 0.;

	hmi.H2_Solomon_dissoc_rate_BigH2_H2s = 0.;
	hmi.H2_Solomon_dissoc_rate_BigH2_H2g = 0.;
	hmi.H2_H2g_to_H2s_rate_BigH2 = 0.;
	hmi.H2_photodissoc_BigH2_H2s = 0.;
	hmi.H2_photodissoc_BigH2_H2g = 0.;

	/* say that H2 has never been computed */
	hmi.lgBigH2_evaluated = FALSE;

	hmi.lgH2_Thermal_BigH2 = TRUE;
	hmi.lgH2_Chemistry_BigH2 = TRUE;

	if( !lgH2_READ_DATA )
	{
		/* the number of electronic levels in the H2 molecule,
		 * to just do the Lyman and Werner bands set to 3 -
		 * reset with atom h2 levels command,
		 * default is all levels with data */
		mole.n_h2_elec_states = N_H2_ELEC;
	}

	/* the number of levels used in the matrix solution
	 * of the level H2_populations - set with atom h2 matrix nlevel,
	 * >>chng 04 oct 05, make default 30 levels 
	 * >>chng 04 dec 23, make default 70 levels */
	nXLevelsMatrix = 70;

	/* this is used to establish zone number for evaluation of number of levels in matrix */
	nzone_nlevel_set = -1;

	nzoneAsEval = -1;
	iterationAsEval = -1; 

#	ifdef DEBUG_FUN
	fputs( " <->H2_Zero()\n", debug_fp );
#	endif

	return;
}
